# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common_types/common_types.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x63ommon_types/common_types.proto\x12\x0fpb_common_types\"{\n\x11ProcedureProgress\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x10\n\x08progress\x18\x03 \x01(\x02\x12\x11\n\tcompleted\x18\x04 \x01(\x08\x12\x14\n\x07\x64\x65tails\x18\x05 \x01(\tH\x00\x88\x01\x01\x42\n\n\x08_details\"\x83\x01\n\x0bJobProgress\x12\x10\n\x08job_uuid\x18\x01 \x01(\t\x12*\n\x06status\x18\x02 \x01(\x0e\x32\x1a.pb_common_types.JobStatus\x12\x36\n\nprogresses\x18\x03 \x03(\x0b\x32\".pb_common_types.ProcedureProgress\"\x8d\x01\n\nStacktrace\x12\x31\n\x06\x66rames\x18\x01 \x03(\x0b\x32!.pb_common_types.Stacktrace.Frame\x1aL\n\x05\x46rame\x12\x17\n\x0fsource_location\x18\x01 \x01(\t\x12\x13\n\x0bsource_line\x18\x02 \x01(\x04\x12\x15\n\rfunction_name\x18\x03 \x01(\t\"\x93\x01\n\x0cJobErrorInfo\x12\x0c\n\x04what\x18\x01 \x01(\t\x12\x1c\n\x0f\x61\x64\x64itional_info\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x34\n\nstacktrace\x18\x03 \x01(\x0b\x32\x1b.pb_common_types.StacktraceH\x01\x88\x01\x01\x42\x12\n\x10_additional_infoB\r\n\x0b_stacktrace*Y\n\tJobStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x12\x0b\n\x07PRE_JOB\x10\x02\x12\x0b\n\x07READ_DB\x10\x03\x12\x0b\n\x07\x43OMPUTE\x10\x04\x12\r\n\tCOMPLETED\x10\x05*\xf5\x01\n\x11\x43omputationMethod\x12\"\n\x1e\x43OMPUTATION_METHOD_UNSPECIFIED\x10\x00\x12\x1b\n\x17\x43OMPUTATION_METHOD_MEAN\x10\x01\x12\x1f\n\x1b\x43OMPUTATION_METHOD_VARIANCE\x10\x02\x12\x1a\n\x16\x43OMPUTATION_METHOD_SUM\x10\x03\x12\x1d\n\x19\x43OMPUTATION_METHOD_CORREL\x10\x04\x12 \n\x1c\x43OMPUTATION_METHOD_MESH_CODE\x10\x05\x12!\n\x1d\x43OMPUTATION_METHOD_JOIN_TABLE\x10\x06\x42\x39Z7github.com/acompany-develop/QuickMPC/proto/common_typesb\x06proto3')

_JOBSTATUS = DESCRIPTOR.enum_types_by_name['JobStatus']
JobStatus = enum_type_wrapper.EnumTypeWrapper(_JOBSTATUS)
_COMPUTATIONMETHOD = DESCRIPTOR.enum_types_by_name['ComputationMethod']
ComputationMethod = enum_type_wrapper.EnumTypeWrapper(_COMPUTATIONMETHOD)
UNKNOWN = 0
ERROR = 1
PRE_JOB = 2
READ_DB = 3
COMPUTE = 4
COMPLETED = 5
COMPUTATION_METHOD_UNSPECIFIED = 0
COMPUTATION_METHOD_MEAN = 1
COMPUTATION_METHOD_VARIANCE = 2
COMPUTATION_METHOD_SUM = 3
COMPUTATION_METHOD_CORREL = 4
COMPUTATION_METHOD_MESH_CODE = 5
COMPUTATION_METHOD_JOIN_TABLE = 6


_PROCEDUREPROGRESS = DESCRIPTOR.message_types_by_name['ProcedureProgress']
_JOBPROGRESS = DESCRIPTOR.message_types_by_name['JobProgress']
_STACKTRACE = DESCRIPTOR.message_types_by_name['Stacktrace']
_STACKTRACE_FRAME = _STACKTRACE.nested_types_by_name['Frame']
_JOBERRORINFO = DESCRIPTOR.message_types_by_name['JobErrorInfo']
ProcedureProgress = _reflection.GeneratedProtocolMessageType('ProcedureProgress', (_message.Message,), {
    'DESCRIPTOR': _PROCEDUREPROGRESS,
    '__module__': 'common_types.common_types_pb2'
    # @@protoc_insertion_point(class_scope:pb_common_types.ProcedureProgress)
})
_sym_db.RegisterMessage(ProcedureProgress)

JobProgress = _reflection.GeneratedProtocolMessageType('JobProgress', (_message.Message,), {
    'DESCRIPTOR': _JOBPROGRESS,
    '__module__': 'common_types.common_types_pb2'
    # @@protoc_insertion_point(class_scope:pb_common_types.JobProgress)
})
_sym_db.RegisterMessage(JobProgress)

Stacktrace = _reflection.GeneratedProtocolMessageType('Stacktrace', (_message.Message,), {

    'Frame': _reflection.GeneratedProtocolMessageType('Frame', (_message.Message,), {
        'DESCRIPTOR': _STACKTRACE_FRAME,
        '__module__': 'common_types.common_types_pb2'
        # @@protoc_insertion_point(class_scope:pb_common_types.Stacktrace.Frame)
    }),
    'DESCRIPTOR': _STACKTRACE,
    '__module__': 'common_types.common_types_pb2'
    # @@protoc_insertion_point(class_scope:pb_common_types.Stacktrace)
})
_sym_db.RegisterMessage(Stacktrace)
_sym_db.RegisterMessage(Stacktrace.Frame)

JobErrorInfo = _reflection.GeneratedProtocolMessageType('JobErrorInfo', (_message.Message,), {
    'DESCRIPTOR': _JOBERRORINFO,
    '__module__': 'common_types.common_types_pb2'
    # @@protoc_insertion_point(class_scope:pb_common_types.JobErrorInfo)
})
_sym_db.RegisterMessage(JobErrorInfo)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'Z7github.com/acompany-develop/QuickMPC/proto/common_types'
    _JOBSTATUS._serialized_start = 605
    _JOBSTATUS._serialized_end = 694
    _COMPUTATIONMETHOD._serialized_start = 697
    _COMPUTATIONMETHOD._serialized_end = 942
    _PROCEDUREPROGRESS._serialized_start = 52
    _PROCEDUREPROGRESS._serialized_end = 175
    _JOBPROGRESS._serialized_start = 178
    _JOBPROGRESS._serialized_end = 309
    _STACKTRACE._serialized_start = 312
    _STACKTRACE._serialized_end = 453
    _STACKTRACE_FRAME._serialized_start = 377
    _STACKTRACE_FRAME._serialized_end = 453
    _JOBERRORINFO._serialized_start = 456
    _JOBERRORINFO._serialized_end = 603
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: libc_to_manage.proto
"""Generated protocol buffer code."""
from common_types import common_types_pb2 as common__types_dot_common__types__pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14libc_to_manage.proto\x12\x0clibctomanage\x1a\x1f\x63ommon_types/common_types.proto\"\x8f\x01\n\x11SendSharesRequest\x12\x0f\n\x07\x64\x61ta_id\x18\x01 \x01(\t\x12\x0e\n\x06shares\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x03(\t\x12\x10\n\x08piece_id\x18\x04 \x01(\x05\x12\x0f\n\x07sent_at\x18\x05 \x01(\t\x12\x17\n\x0fmatching_column\x18\x06 \x01(\x05\x12\r\n\x05token\x18\x07 \x01(\t\"4\n\x12SendSharesResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\r\n\x05is_ok\x18\x02 \x01(\x08\"5\n\x13\x44\x65leteSharesRequest\x12\x0f\n\x07\x64\x61taIds\x18\x01 \x03(\t\x12\r\n\x05token\x18\x02 \x01(\t\"6\n\x14\x44\x65leteSharesResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\r\n\x05is_ok\x18\x02 \x01(\x08\"2\n\x10GetSchemaRequest\x12\x0f\n\x07\x64\x61ta_id\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\t\"C\n\x11GetSchemaResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\r\n\x05is_ok\x18\x02 \x01(\x08\x12\x0e\n\x06schema\x18\x03 \x03(\t\"9\n\tJoinOrder\x12\x0f\n\x07\x64\x61taIds\x18\x01 \x03(\t\x12\x0c\n\x04join\x18\x02 \x03(\x05\x12\r\n\x05index\x18\x03 \x03(\x05\"$\n\x05Input\x12\x0b\n\x03src\x18\x01 \x03(\x05\x12\x0e\n\x06target\x18\x02 \x03(\x05\"\xab\x01\n\x19\x45xecuteComputationRequest\x12\x35\n\tmethod_id\x18\x01 \x01(\x0e\x32\".pb_common_types.ComputationMethod\x12\r\n\x05token\x18\x02 \x01(\t\x12&\n\x05table\x18\x03 \x01(\x0b\x32\x17.libctomanage.JoinOrder\x12 \n\x03\x61rg\x18\x04 \x01(\x0b\x32\x13.libctomanage.Input\"N\n\x1a\x45xecuteComputationResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\r\n\x05is_ok\x18\x02 \x01(\x08\x12\x10\n\x08job_uuid\x18\x03 \x01(\t\">\n\x1bGetComputationResultRequest\x12\x10\n\x08job_uuid\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\t\"\xaf\x02\n\x1cGetComputationResultResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\r\n\x05is_ok\x18\x02 \x01(\x08\x12\x0e\n\x06result\x18\x03 \x03(\t\x12\x15\n\rcolumn_number\x18\x04 \x01(\x05\x12*\n\x06status\x18\x05 \x01(\x0e\x32\x1a.pb_common_types.JobStatus\x12\x10\n\x08piece_id\x18\x06 \x01(\x05\x12\x33\n\x08progress\x18\x07 \x01(\x0b\x32\x1c.pb_common_types.JobProgressH\x01\x88\x01\x01\x12\x11\n\x07is_dim1\x18\x08 \x01(\x08H\x00\x12\x11\n\x07is_dim2\x18\t \x01(\x08H\x00\x12\x13\n\tis_schema\x18\n \x01(\x08H\x00\x42\r\n\x0bresult_typeB\x0b\n\t_progress\"#\n\x12GetDataListRequest\x12\r\n\x05token\x18\x01 \x01(\t\"4\n\x13GetDataListResponse\x12\x0e\n\x06result\x18\x01 \x01(\t\x12\r\n\x05is_ok\x18\x02 \x01(\x08\"8\n\x15GetElapsedTimeRequest\x12\x10\n\x08job_uuid\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\t\"=\n\x16GetElapsedTimeResponse\x12\r\n\x05is_ok\x18\x01 \x01(\x08\x12\x14\n\x0c\x65lapsed_time\x18\x02 \x01(\x01\"9\n\x16GetJobErrorInfoRequest\x12\x10\n\x08job_uuid\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\t\"w\n\x17GetJobErrorInfoResponse\x12:\n\x0ejob_error_info\x18\x01 \x01(\x0b\x32\x1d.pb_common_types.JobErrorInfoH\x00\x88\x01\x01\x12\r\n\x05is_ok\x18\x02 \x01(\x08\x42\x11\n\x0f_job_error_info2\xff\x05\n\x0cLibcToManage\x12Q\n\nSendShares\x12\x1f.libctomanage.SendSharesRequest\x1a .libctomanage.SendSharesResponse\"\x00\x12W\n\x0c\x44\x65leteShares\x12!.libctomanage.DeleteSharesRequest\x1a\".libctomanage.DeleteSharesResponse\"\x00\x12N\n\tGetSchema\x12\x1e.libctomanage.GetSchemaRequest\x1a\x1f.libctomanage.GetSchemaResponse\"\x00\x12i\n\x12\x45xecuteComputation\x12\'.libctomanage.ExecuteComputationRequest\x1a(.libctomanage.ExecuteComputationResponse\"\x00\x12q\n\x14GetComputationResult\x12).libctomanage.GetComputationResultRequest\x1a*.libctomanage.GetComputationResultResponse\"\x00\x30\x01\x12T\n\x0bGetDataList\x12 .libctomanage.GetDataListRequest\x1a!.libctomanage.GetDataListResponse\"\x00\x12]\n\x0eGetElapsedTime\x12#.libctomanage.GetElapsedTimeRequest\x1a$.libctomanage.GetElapsedTimeResponse\"\x00\x12`\n\x0fGetJobErrorInfo\x12$.libctomanage.GetJobErrorInfoRequest\x1a%.libctomanage.GetJobErrorInfoResponse\"\x00\x42\x45ZCgithub.com/acompany-develop/QuickMPC/proto/libc_to_manage_containerb\x06proto3')


_SENDSHARESREQUEST = DESCRIPTOR.message_types_by_name['SendSharesRequest']
_SENDSHARESRESPONSE = DESCRIPTOR.message_types_by_name['SendSharesResponse']
_DELETESHARESREQUEST = DESCRIPTOR.message_types_by_name['DeleteSharesRequest']
_DELETESHARESRESPONSE = DESCRIPTOR.message_types_by_name['DeleteSharesResponse']
_GETSCHEMAREQUEST = DESCRIPTOR.message_types_by_name['GetSchemaRequest']
_GETSCHEMARESPONSE = DESCRIPTOR.message_types_by_name['GetSchemaResponse']
_JOINORDER = DESCRIPTOR.message_types_by_name['JoinOrder']
_INPUT = DESCRIPTOR.message_types_by_name['Input']
_EXECUTECOMPUTATIONREQUEST = DESCRIPTOR.message_types_by_name['ExecuteComputationRequest']
_EXECUTECOMPUTATIONRESPONSE = DESCRIPTOR.message_types_by_name['ExecuteComputationResponse']
_GETCOMPUTATIONRESULTREQUEST = DESCRIPTOR.message_types_by_name['GetComputationResultRequest']
_GETCOMPUTATIONRESULTRESPONSE = DESCRIPTOR.message_types_by_name['GetComputationResultResponse']
_GETDATALISTREQUEST = DESCRIPTOR.message_types_by_name['GetDataListRequest']
_GETDATALISTRESPONSE = DESCRIPTOR.message_types_by_name['GetDataListResponse']
_GETELAPSEDTIMEREQUEST = DESCRIPTOR.message_types_by_name['GetElapsedTimeRequest']
_GETELAPSEDTIMERESPONSE = DESCRIPTOR.message_types_by_name['GetElapsedTimeResponse']
_GETJOBERRORINFOREQUEST = DESCRIPTOR.message_types_by_name['GetJobErrorInfoRequest']
_GETJOBERRORINFORESPONSE = DESCRIPTOR.message_types_by_name['GetJobErrorInfoResponse']
SendSharesRequest = _reflection.GeneratedProtocolMessageType('SendSharesRequest', (_message.Message,), {
    'DESCRIPTOR': _SENDSHARESREQUEST,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.SendSharesRequest)
})
_sym_db.RegisterMessage(SendSharesRequest)

SendSharesResponse = _reflection.GeneratedProtocolMessageType('SendSharesResponse', (_message.Message,), {
    'DESCRIPTOR': _SENDSHARESRESPONSE,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.SendSharesResponse)
})
_sym_db.RegisterMessage(SendSharesResponse)

DeleteSharesRequest = _reflection.GeneratedProtocolMessageType('DeleteSharesRequest', (_message.Message,), {
    'DESCRIPTOR': _DELETESHARESREQUEST,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.DeleteSharesRequest)
})
_sym_db.RegisterMessage(DeleteSharesRequest)

DeleteSharesResponse = _reflection.GeneratedProtocolMessageType('DeleteSharesResponse', (_message.Message,), {
    'DESCRIPTOR': _DELETESHARESRESPONSE,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.DeleteSharesResponse)
})
_sym_db.RegisterMessage(DeleteSharesResponse)

GetSchemaRequest = _reflection.GeneratedProtocolMessageType('GetSchemaRequest', (_message.Message,), {
    'DESCRIPTOR': _GETSCHEMAREQUEST,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetSchemaRequest)
})
_sym_db.RegisterMessage(GetSchemaRequest)

GetSchemaResponse = _reflection.GeneratedProtocolMessageType('GetSchemaResponse', (_message.Message,), {
    'DESCRIPTOR': _GETSCHEMARESPONSE,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetSchemaResponse)
})
_sym_db.RegisterMessage(GetSchemaResponse)

JoinOrder = _reflection.GeneratedProtocolMessageType('JoinOrder', (_message.Message,), {
    'DESCRIPTOR': _JOINORDER,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.JoinOrder)
})
_sym_db.RegisterMessage(JoinOrder)

Input = _reflection.GeneratedProtocolMessageType('Input', (_message.Message,), {
    'DESCRIPTOR': _INPUT,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.Input)
})
_sym_db.RegisterMessage(Input)

ExecuteComputationRequest = _reflection.GeneratedProtocolMessageType('ExecuteComputationRequest', (_message.Message,), {
    'DESCRIPTOR': _EXECUTECOMPUTATIONREQUEST,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.ExecuteComputationRequest)
})
_sym_db.RegisterMessage(ExecuteComputationRequest)

ExecuteComputationResponse = _reflection.GeneratedProtocolMessageType('ExecuteComputationResponse', (_message.Message,), {
    'DESCRIPTOR': _EXECUTECOMPUTATIONRESPONSE,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.ExecuteComputationResponse)
})
_sym_db.RegisterMessage(ExecuteComputationResponse)

GetComputationResultRequest = _reflection.GeneratedProtocolMessageType('GetComputationResultRequest', (_message.Message,), {
    'DESCRIPTOR': _GETCOMPUTATIONRESULTREQUEST,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetComputationResultRequest)
})
_sym_db.RegisterMessage(GetComputationResultRequest)

GetComputationResultResponse = _reflection.GeneratedProtocolMessageType('GetComputationResultResponse', (_message.Message,), {
    'DESCRIPTOR': _GETCOMPUTATIONRESULTRESPONSE,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetComputationResultResponse)
})
_sym_db.RegisterMessage(GetComputationResultResponse)

GetDataListRequest = _reflection.GeneratedProtocolMessageType('GetDataListRequest', (_message.Message,), {
    'DESCRIPTOR': _GETDATALISTREQUEST,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetDataListRequest)
})
_sym_db.RegisterMessage(GetDataListRequest)

GetDataListResponse = _reflection.GeneratedProtocolMessageType('GetDataListResponse', (_message.Message,), {
    'DESCRIPTOR': _GETDATALISTRESPONSE,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetDataListResponse)
})
_sym_db.RegisterMessage(GetDataListResponse)

GetElapsedTimeRequest = _reflection.GeneratedProtocolMessageType('GetElapsedTimeRequest', (_message.Message,), {
    'DESCRIPTOR': _GETELAPSEDTIMEREQUEST,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetElapsedTimeRequest)
})
_sym_db.RegisterMessage(GetElapsedTimeRequest)

GetElapsedTimeResponse = _reflection.GeneratedProtocolMessageType('GetElapsedTimeResponse', (_message.Message,), {
    'DESCRIPTOR': _GETELAPSEDTIMERESPONSE,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetElapsedTimeResponse)
})
_sym_db.RegisterMessage(GetElapsedTimeResponse)

GetJobErrorInfoRequest = _reflection.GeneratedProtocolMessageType('GetJobErrorInfoRequest', (_message.Message,), {
    'DESCRIPTOR': _GETJOBERRORINFOREQUEST,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetJobErrorInfoRequest)
})
_sym_db.RegisterMessage(GetJobErrorInfoRequest)

GetJobErrorInfoResponse = _reflection.GeneratedProtocolMessageType('GetJobErrorInfoResponse', (_message.Message,), {
    'DESCRIPTOR': _GETJOBERRORINFORESPONSE,
    '__module__': 'libc_to_manage_pb2'
    # @@protoc_insertion_point(class_scope:libctomanage.GetJobErrorInfoResponse)
})
_sym_db.RegisterMessage(GetJobErrorInfoResponse)

_LIBCTOMANAGE = DESCRIPTOR.services_by_name['LibcToManage']
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'ZCgithub.com/acompany-develop/QuickMPC/proto/libc_to_manage_container'
    _SENDSHARESREQUEST._serialized_start = 72
    _SENDSHARESREQUEST._serialized_end = 215
    _SENDSHARESRESPONSE._serialized_start = 217
    _SENDSHARESRESPONSE._serialized_end = 269
    _DELETESHARESREQUEST._serialized_start = 271
    _DELETESHARESREQUEST._serialized_end = 324
    _DELETESHARESRESPONSE._serialized_start = 326
    _DELETESHARESRESPONSE._serialized_end = 380
    _GETSCHEMAREQUEST._serialized_start = 382
    _GETSCHEMAREQUEST._serialized_end = 432
    _GETSCHEMARESPONSE._serialized_start = 434
    _GETSCHEMARESPONSE._serialized_end = 501
    _JOINORDER._serialized_start = 503
    _JOINORDER._serialized_end = 560
    _INPUT._serialized_start = 562
    _INPUT._serialized_end = 598
    _EXECUTECOMPUTATIONREQUEST._serialized_start = 601
    _EXECUTECOMPUTATIONREQUEST._serialized_end = 772
    _EXECUTECOMPUTATIONRESPONSE._serialized_start = 774
    _EXECUTECOMPUTATIONRESPONSE._serialized_end = 852
    _GETCOMPUTATIONRESULTREQUEST._serialized_start = 854
    _GETCOMPUTATIONRESULTREQUEST._serialized_end = 916
    _GETCOMPUTATIONRESULTRESPONSE._serialized_start = 919
    _GETCOMPUTATIONRESULTRESPONSE._serialized_end = 1222
    _GETDATALISTREQUEST._serialized_start = 1224
    _GETDATALISTREQUEST._serialized_end = 1259
    _GETDATALISTRESPONSE._serialized_start = 1261
    _GETDATALISTRESPONSE._serialized_end = 1313
    _GETELAPSEDTIMEREQUEST._serialized_start = 1315
    _GETELAPSEDTIMEREQUEST._serialized_end = 1371
    _GETELAPSEDTIMERESPONSE._serialized_start = 1373
    _GETELAPSEDTIMERESPONSE._serialized_end = 1434
    _GETJOBERRORINFOREQUEST._serialized_start = 1436
    _GETJOBERRORINFOREQUEST._serialized_end = 1493
    _GETJOBERRORINFORESPONSE._serialized_start = 1495
    _GETJOBERRORINFORESPONSE._serialized_end = 1614
    _LIBCTOMANAGE._serialized_start = 1617
    _LIBCTOMANAGE._serialized_end = 2384
# @@protoc_insertion_point(module_scope)



from quickmpc.utils.overload_tools import (DictList, DictList2, Dim1, Dim2,
                                           Dim3, methoddispatch)


def test_overload():
    """methodが正しくoverloadされるかTest"""

    class Class:
        @methoddispatch()
        def method(self, val):
            return 0

        @method.register(int)
        def method1(self, val):
            return 1

        @method.register(float)
        def method2(self, val):
            return 2

        @method.register(str)
        def method3(self, val):
            return 3

        @method.register(list)
        def method4(self, val):
            return 4

    d: Class = Class()
    assert (d.method({}) == 0)
    assert (d.method(0) == 1)
    assert (d.method(0.0) == 2)
    assert (d.method("0") == 3)
    assert (d.method([0]) == 4)


def test_overload_static():
    """stataic methodが正しくoverloadされるかTest"""
    class Class:
        @methoddispatch(is_static_method=True)
        @staticmethod
        def method(val):
            return 0

        @method.register(int)
        @staticmethod
        def method1(val):
            return 1

        @method.register(float)
        @staticmethod
        def method2(val):
            return 2

        @method.register(str)
        @staticmethod
        def method3(val):
            return 3

        @method.register(list)
        @staticmethod
        def method4(val):
            return 4

    assert (Class.method({}) == 0)
    assert (Class.method(0) == 1)
    assert (Class.method(0.0) == 2)
    assert (Class.method("0") == 3)
    assert (Class.method([0]) == 4)


def test_overload_dim():
    """methodが次元数に応じて正しくoverloadされるかTest"""

    class Class:
        @methoddispatch()
        def method(self, val):
            return 0

        @method.register(int)
        def method1(self, val):
            return 1

        @method.register(Dim1)
        def method2(self, val):
            return 2

        @method.register(Dim2)
        def method3(self, val):
            return 3

        @method.register(Dim3)
        def method4(self, val):
            return 4

        @method.register(DictList)
        def method5(self, val):
            return 5

        @method.register(DictList2)
        def method6(self, val):
            return 6

    d: Class = Class()
    assert (d.method({}) == 0)
    assert (d.method(0) == 1)
    assert (d.method([0]) == 2)
    assert (d.method([[0]]) == 3)
    assert (d.method([[[0]]]) == 4)
    assert (d.method([{}]) == 5)
    assert (d.method([[{}]]) == 6)
    assert (d.method([[[[0]]]]) == 0)



import sys
import logging

from quickmpc import QMPC

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG)

if __name__ == '__main__':
    # QuickMPC Setting
    qmpc: QMPC = QMPC(
        ["http://localhost:50001",
         "http://localhost:50002",
         "http://localhost:50003"]
    )

    if not len(sys.argv) == 2:
        print('Usage: $ pipenv run python send_share.py [file_path]',
              file=sys.stderr)
        sys.exit(1)
    # csv dataをパースする
    filename: str = sys.argv[1]

    secrets, schema = qmpc.parse_csv_file(filename)

    # データをシェア化し送信する
    res = qmpc.send_share(secrets, schema)
    logger.info(res)



from dataclasses import dataclass
from typing import Union
from google.protobuf.reflection import GeneratedProtocolMessageType


@dataclass(frozen=True)
class QMPCJobError(Exception):
    err_info: GeneratedProtocolMessageType


@dataclass(frozen=True)
class QMPCServerError(Exception):
    err_info: Union[str, GeneratedProtocolMessageType]


class ArgmentError(Exception):
    pass



from decimal import Decimal

import pytest

from quickmpc.utils.random import ChaCha20, RandomInterface


class TestCsprng:
    rnd: RandomInterface = ChaCha20()

    @pytest.mark.parametrize(
        ("lower", "upper"),
        [
            (0, 10),  # 正のみ
            (-5, 5),  # 正負
            (-10, 0),  # 負のみ
            (0, 1),   # 範囲が1
            (-(1 << 128), -(1 << 128) + 1),  # 128bit最小値
            ((1 << 128) - 1, (1 << 128) + 1)  # 128bit最大値
        ]
    )
    def test_csprng_int_interval(self, lower: int, upper: int):
        """ 半開区間内で生成されるかのtest """
        exist_lower: bool = False
        exist_upper_1: bool = False
        for _ in range(1000):
            x: int = self.rnd.get(lower, upper)
            assert (lower <= x and x < upper)
            assert (isinstance(x, int))
            exist_lower |= (x == lower)
            exist_upper_1 |= (x == upper - 1)
        for x in self.rnd.get_list(lower, upper, 1000):
            assert (lower <= x and x < upper)
            assert (isinstance(x, int))
            exist_lower |= (x == lower)
            exist_upper_1 |= (x == upper - 1)
        assert (exist_lower)
        assert (exist_upper_1)

    @pytest.mark.parametrize(
        ("lower", "upper"),
        [
            (Decimal(0.0), Decimal(10.0)),  # 正のみ
            (Decimal(-5.0), Decimal(5.0)),  # 正負
            (Decimal(-10.0), Decimal(0.0)),  # 負のみ
            (Decimal(0.0), Decimal(1.0)),   # 範囲が1
            (Decimal(-(1 << 64)), Decimal(-(1 << 64) + 1)),
            (Decimal((1 << 64) - 1), Decimal((1 << 64) + 1))
        ]
    )
    def test_csprng_decimal_interval(self, lower: Decimal, upper: Decimal):
        """ 半開区間内で生成されるかのtest """
        for _ in range(1000):
            x: int = self.rnd.get(lower, upper)
            assert (lower <= x and x < upper)
            assert (isinstance(x, Decimal))
        for x in self.rnd.get_list(lower, upper, 1000):
            assert (lower <= x and x < upper)
            assert (isinstance(x, Decimal))

    @pytest.mark.parametrize(
        ("lower", "upper"),
        [
            (0, 0),
            (10, 0),
            (0, 1.2),
            (0.0, 1),
            ("hoge", "huga"),
        ]
    )
    def test_csprng_errorhandring(self, lower, upper):
        """ 異常値を与えてエラーが出るかtest """
        with pytest.raises(Exception):
            self.rnd.get(lower, upper)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import libc_to_manage_pb2 as libc__to__manage__pb2


class LibcToManageStub(object):
    """*
    LibcToManage service
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SendShares = channel.unary_unary(
            '/libctomanage.LibcToManage/SendShares',
            request_serializer=libc__to__manage__pb2.SendSharesRequest.SerializeToString,
            response_deserializer=libc__to__manage__pb2.SendSharesResponse.FromString,
        )
        self.DeleteShares = channel.unary_unary(
            '/libctomanage.LibcToManage/DeleteShares',
            request_serializer=libc__to__manage__pb2.DeleteSharesRequest.SerializeToString,
            response_deserializer=libc__to__manage__pb2.DeleteSharesResponse.FromString,
        )
        self.GetSchema = channel.unary_unary(
            '/libctomanage.LibcToManage/GetSchema',
            request_serializer=libc__to__manage__pb2.GetSchemaRequest.SerializeToString,
            response_deserializer=libc__to__manage__pb2.GetSchemaResponse.FromString,
        )
        self.ExecuteComputation = channel.unary_unary(
            '/libctomanage.LibcToManage/ExecuteComputation',
            request_serializer=libc__to__manage__pb2.ExecuteComputationRequest.SerializeToString,
            response_deserializer=libc__to__manage__pb2.ExecuteComputationResponse.FromString,
        )
        self.GetComputationResult = channel.unary_stream(
            '/libctomanage.LibcToManage/GetComputationResult',
            request_serializer=libc__to__manage__pb2.GetComputationResultRequest.SerializeToString,
            response_deserializer=libc__to__manage__pb2.GetComputationResultResponse.FromString,
        )
        self.GetDataList = channel.unary_unary(
            '/libctomanage.LibcToManage/GetDataList',
            request_serializer=libc__to__manage__pb2.GetDataListRequest.SerializeToString,
            response_deserializer=libc__to__manage__pb2.GetDataListResponse.FromString,
        )
        self.GetElapsedTime = channel.unary_unary(
            '/libctomanage.LibcToManage/GetElapsedTime',
            request_serializer=libc__to__manage__pb2.GetElapsedTimeRequest.SerializeToString,
            response_deserializer=libc__to__manage__pb2.GetElapsedTimeResponse.FromString,
        )
        self.GetJobErrorInfo = channel.unary_unary(
            '/libctomanage.LibcToManage/GetJobErrorInfo',
            request_serializer=libc__to__manage__pb2.GetJobErrorInfoRequest.SerializeToString,
            response_deserializer=libc__to__manage__pb2.GetJobErrorInfoResponse.FromString,
        )


class LibcToManageServicer(object):
    """*
    LibcToManage service
    """

    def SendShares(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteShares(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSchema(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExecuteComputation(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetComputationResult(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDataList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetElapsedTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetJobErrorInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LibcToManageServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'SendShares': grpc.unary_unary_rpc_method_handler(
            servicer.SendShares,
            request_deserializer=libc__to__manage__pb2.SendSharesRequest.FromString,
            response_serializer=libc__to__manage__pb2.SendSharesResponse.SerializeToString,
        ),
        'DeleteShares': grpc.unary_unary_rpc_method_handler(
            servicer.DeleteShares,
            request_deserializer=libc__to__manage__pb2.DeleteSharesRequest.FromString,
            response_serializer=libc__to__manage__pb2.DeleteSharesResponse.SerializeToString,
        ),
        'GetSchema': grpc.unary_unary_rpc_method_handler(
            servicer.GetSchema,
            request_deserializer=libc__to__manage__pb2.GetSchemaRequest.FromString,
            response_serializer=libc__to__manage__pb2.GetSchemaResponse.SerializeToString,
        ),
        'ExecuteComputation': grpc.unary_unary_rpc_method_handler(
            servicer.ExecuteComputation,
            request_deserializer=libc__to__manage__pb2.ExecuteComputationRequest.FromString,
            response_serializer=libc__to__manage__pb2.ExecuteComputationResponse.SerializeToString,
        ),
        'GetComputationResult': grpc.unary_stream_rpc_method_handler(
            servicer.GetComputationResult,
            request_deserializer=libc__to__manage__pb2.GetComputationResultRequest.FromString,
            response_serializer=libc__to__manage__pb2.GetComputationResultResponse.SerializeToString,
        ),
        'GetDataList': grpc.unary_unary_rpc_method_handler(
            servicer.GetDataList,
            request_deserializer=libc__to__manage__pb2.GetDataListRequest.FromString,
            response_serializer=libc__to__manage__pb2.GetDataListResponse.SerializeToString,
        ),
        'GetElapsedTime': grpc.unary_unary_rpc_method_handler(
            servicer.GetElapsedTime,
            request_deserializer=libc__to__manage__pb2.GetElapsedTimeRequest.FromString,
            response_serializer=libc__to__manage__pb2.GetElapsedTimeResponse.SerializeToString,
        ),
        'GetJobErrorInfo': grpc.unary_unary_rpc_method_handler(
            servicer.GetJobErrorInfo,
            request_deserializer=libc__to__manage__pb2.GetJobErrorInfoRequest.FromString,
            response_serializer=libc__to__manage__pb2.GetJobErrorInfoResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'libctomanage.LibcToManage', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))

 # This class is part of an EXPERIMENTAL API.


class LibcToManage(object):
    """*
    LibcToManage service
    """

    @staticmethod
    def SendShares(request,
                   target,
                   options=(),
                   channel_credentials=None,
                   call_credentials=None,
                   insecure=False,
                   compression=None,
                   wait_for_ready=None,
                   timeout=None,
                   metadata=None):
        return grpc.experimental.unary_unary(request, target, '/libctomanage.LibcToManage/SendShares',
                                             libc__to__manage__pb2.SendSharesRequest.SerializeToString,
                                             libc__to__manage__pb2.SendSharesResponse.FromString,
                                             options, channel_credentials,
                                             insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteShares(request,
                     target,
                     options=(),
                     channel_credentials=None,
                     call_credentials=None,
                     insecure=False,
                     compression=None,
                     wait_for_ready=None,
                     timeout=None,
                     metadata=None):
        return grpc.experimental.unary_unary(request, target, '/libctomanage.LibcToManage/DeleteShares',
                                             libc__to__manage__pb2.DeleteSharesRequest.SerializeToString,
                                             libc__to__manage__pb2.DeleteSharesResponse.FromString,
                                             options, channel_credentials,
                                             insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSchema(request,
                  target,
                  options=(),
                  channel_credentials=None,
                  call_credentials=None,
                  insecure=False,
                  compression=None,
                  wait_for_ready=None,
                  timeout=None,
                  metadata=None):
        return grpc.experimental.unary_unary(request, target, '/libctomanage.LibcToManage/GetSchema',
                                             libc__to__manage__pb2.GetSchemaRequest.SerializeToString,
                                             libc__to__manage__pb2.GetSchemaResponse.FromString,
                                             options, channel_credentials,
                                             insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExecuteComputation(request,
                           target,
                           options=(),
                           channel_credentials=None,
                           call_credentials=None,
                           insecure=False,
                           compression=None,
                           wait_for_ready=None,
                           timeout=None,
                           metadata=None):
        return grpc.experimental.unary_unary(request, target, '/libctomanage.LibcToManage/ExecuteComputation',
                                             libc__to__manage__pb2.ExecuteComputationRequest.SerializeToString,
                                             libc__to__manage__pb2.ExecuteComputationResponse.FromString,
                                             options, channel_credentials,
                                             insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetComputationResult(request,
                             target,
                             options=(),
                             channel_credentials=None,
                             call_credentials=None,
                             insecure=False,
                             compression=None,
                             wait_for_ready=None,
                             timeout=None,
                             metadata=None):
        return grpc.experimental.unary_stream(request, target, '/libctomanage.LibcToManage/GetComputationResult',
                                              libc__to__manage__pb2.GetComputationResultRequest.SerializeToString,
                                              libc__to__manage__pb2.GetComputationResultResponse.FromString,
                                              options, channel_credentials,
                                              insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDataList(request,
                    target,
                    options=(),
                    channel_credentials=None,
                    call_credentials=None,
                    insecure=False,
                    compression=None,
                    wait_for_ready=None,
                    timeout=None,
                    metadata=None):
        return grpc.experimental.unary_unary(request, target, '/libctomanage.LibcToManage/GetDataList',
                                             libc__to__manage__pb2.GetDataListRequest.SerializeToString,
                                             libc__to__manage__pb2.GetDataListResponse.FromString,
                                             options, channel_credentials,
                                             insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetElapsedTime(request,
                       target,
                       options=(),
                       channel_credentials=None,
                       call_credentials=None,
                       insecure=False,
                       compression=None,
                       wait_for_ready=None,
                       timeout=None,
                       metadata=None):
        return grpc.experimental.unary_unary(request, target, '/libctomanage.LibcToManage/GetElapsedTime',
                                             libc__to__manage__pb2.GetElapsedTimeRequest.SerializeToString,
                                             libc__to__manage__pb2.GetElapsedTimeResponse.FromString,
                                             options, channel_credentials,
                                             insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetJobErrorInfo(request,
                        target,
                        options=(),
                        channel_credentials=None,
                        call_credentials=None,
                        insecure=False,
                        compression=None,
                        wait_for_ready=None,
                        timeout=None,
                        metadata=None):
        return grpc.experimental.unary_unary(request, target, '/libctomanage.LibcToManage/GetJobErrorInfo',
                                             libc__to__manage__pb2.GetJobErrorInfoRequest.SerializeToString,
                                             libc__to__manage__pb2.GetJobErrorInfoResponse.FromString,
                                             options, channel_credentials,
                                             insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import glob

import autopep8
import pkg_resources
from grpc.tools import protoc

_proto_path: str = "./../../../../../proto"


def generate_files():
    protoc.main(
        (
            "",
            "-I{}".format(pkg_resources.resource_filename('grpc_tools',
                                                          '_proto')),
            f"-I{_proto_path}/libc_to_manage_container",
            f"-I{_proto_path}",
            "--python_out=.",
            "--grpc_python_out=.",
            f"{_proto_path}/libc_to_manage_container/libc_to_manage.proto",
        )
    )
    protoc.main(
        (
            "",
            f"-I{_proto_path}",
            "--python_out=.",
            f"{_proto_path}/common_types/common_types.proto",
        )
    )


def format_files():
    files = glob.glob("./**/*.py", recursive=True)
    for file in files:
        autopep8.main(["autopep8", "--in-place", file])


if __name__ == '__main__':
    print(f"path: `{_proto_path}`")
    generate_files()
    print("generate finished")
    format_files()
    print("format finished")



""" テスト用のgrpc server """
from concurrent import futures

import grpc
from google.protobuf import any_pb2
from google.rpc import code_pb2, status_pb2
from grpc_status import rpc_status

from quickmpc.proto import libc_to_manage_pb2, libc_to_manage_pb2_grpc
from quickmpc.proto.common_types import common_types_pb2
from quickmpc.proto.common_types.common_types_pb2 import (JobErrorInfo,
                                                          Stacktrace)


class LibToManageServiceServicer(libc_to_manage_pb2_grpc.LibcToManageServicer):

    def __init__(self): ...

    def SendShares(self, request, context):
        res = libc_to_manage_pb2.SendSharesResponse(
            message="ok",
            is_ok=True
        )
        return res

    def DeleteShares(self, request, context):
        res = libc_to_manage_pb2.DeleteSharesResponse(
            message="ok",
            is_ok=True
        )
        return res

    def ExecuteComputation(self, request, context):
        if len(request.arg.src) > 0 \
                and request.arg.src[0] == 1000000000:
            # QMPCJobError
            # CC で Job 実行時にエラーが発生していた場合を再現
            detail = any_pb2.Any()
            detail.Pack(
                JobErrorInfo(
                    what="QMPCJobError",
                )
            )
            rich_status = status_pb2.Status(
                code=code_pb2.INVALID_ARGUMENT,
                details=[detail]
            )
            context.abort_with_status(rpc_status.to_status(rich_status))
            return libc_to_manage_pb2.ExecuteComputationResponse()

        if len(request.table.dataIds) > 0 \
                and request.table.dataIds[0] == "UnregisteredDataId":
            # QMPCServerError
            # MC で Internal Server Error が発生している場合を再現
            context.set_code(grpc.StatusCode.UNKNOWN)
            return libc_to_manage_pb2.ExecuteComputationResponse()

        res = libc_to_manage_pb2.ExecuteComputationResponse(
            message="ok",
            is_ok=True,
            job_uuid="jobjobjob"
        )
        return res

    def GetComputationResult(self, request, context):
        yield libc_to_manage_pb2.GetComputationResultResponse(
            message="ok",
            is_ok=True,
            result=["1"],
            status=common_types_pb2.COMPLETED,
            piece_id=1,
            column_number=2,
            is_dim1=True,
        )
        yield libc_to_manage_pb2.GetComputationResultResponse(
            message="ok",
            is_ok=True,
            result=["2"],
            status=common_types_pb2.COMPLETED,
            piece_id=2,
            column_number=2,
            is_dim1=True,
        )

    def GetDataList(self, request, context):
        res = libc_to_manage_pb2.GetDataListResponse(
            is_ok=True,
            result="[]"
        )
        return res

    def GetJoinTable(self, request, context):
        res = libc_to_manage_pb2.GetJoinTableResponse(
            is_ok=True,
            result="['1']",
            schema=["s"]
        )
        return res

    def GetJobErrorInfo(self, request, context):
        res = libc_to_manage_pb2.GetJobErrorInfoResponse(
            is_ok=True,
            job_error_info=JobErrorInfo(
                what="QMPCJobError",
                stacktrace=Stacktrace()
            )
        )
        return res


def serve(num: int):
    """ server setting """
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    libc_to_manage_pb2_grpc.add_LibcToManageServicer_to_server(
        LibToManageServiceServicer(), server)
    server.add_insecure_port('localhost:5000{}'.format(num))

    return server




import logging
import time

from quickmpc import QMPC

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG)


def send_share(filename: str):
    secrets, schema = qmpc.parse_csv_file(filename)
    res = qmpc.send_share(secrets, schema)
    return res["data_id"]


if __name__ == '__main__':
    # QuickMPC Setting
    qmpc: QMPC = QMPC(
        ["http://localhost:50001",
         "http://localhost:50002",
         "http://localhost:50003"]
    )

    # 複数のシェアを作成・送信
    data_id1: str = send_share("../data/data1-1.csv")
    data_id2: str = send_share("../data/data1-2.csv")

    join_order = ([data_id1, data_id2], [2], [1, 1])
    res = qmpc.get_join_table(join_order)

    job_uuid: str = res["job_uuid"]
    for _ in range(100):
        # 計算が終わるか100秒(1秒x100回)経つまで繰り返す
        get_res = qmpc.get_computation_result(job_uuid)
        if get_res["results"] is not None:
            res = get_res["results"]
            break
        time.sleep(1)
    logger.info(res)



import logging

from quickmpc import QMPC

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG)

if __name__ == '__main__':
    # QuickMPC Setting
    qmpc: QMPC = QMPC(
        ["http://localhost:50001",
         "http://localhost:50002",
         "http://localhost:50003"]
    )

    res = qmpc.get_data_list()
    logger.info(res)



import logging

from quickmpc import QMPC

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG)

if __name__ == '__main__':
    # QuickMPC Setting
    qmpc: QMPC = QMPC(
        ["http://localhost:50001",
         "http://localhost:50002",
         "http://localhost:50003"]
    )

    # data_idに対して計算のリクエストを行う
    data_ids = \
        ["95ac225f2f9c4d72f95c85373fd7ade9d3c22520654f45be569ae9c06c801709"]
    res = qmpc.correl(
        (
            data_ids,  # 結合するdata_ids
            [],  # 結合方向(0:横，1:縦)
            [1]  # 各dataで結合に用いる列
        ),
        (
            [2, 3],  # 指定する列1
            [4]  # 指定する列2(func(列1, 列2)のように計算される)
        )
    )
    logger.info(res)



"""
listの次元によるオーバロードが可能となるような自作dispatch
"""

from functools import singledispatch, update_wrapper


class Dim1:
    ...


class Dim2:
    ...


class Dim3:
    ...


class DictList:
    ...


class DictList2:
    ...


def d(lst):
    if type(lst) != list:
        return 0
    if not lst:
        return 1
    return 1+d(lst[0])


def _get_dim_class(lst: list):
    dim: int = d(lst)
    if dim == 1:
        if len(lst) and type(lst[0]) == dict:
            return DictList()
        return Dim1()
    elif dim == 2:
        if len(lst) and len(lst[0]) and type(lst[0][0]) == dict:
            return DictList2()
        return Dim2()
    elif dim == 3:
        return Dim3()
    return None


def _convert_list_type(registry, type):
    if type.__class__ is not list:
        return type
    for Dim in (Dim1, Dim2, Dim3, DictList):
        if Dim in registry:
            return _get_dim_class(type)
    return type


def methoddispatch(is_static_method: bool = False):
    def _dimdispatch(func):
        dispatcher = singledispatch(func)

        def wrapper(*args, **kw):
            arg = args[0] if is_static_method else args[1]
            type = _convert_list_type(dispatcher.registry, arg)
            if is_static_method:
                return dispatcher.dispatch(
                    type.__class__
                ).__func__(*args, **kw)
            else:
                return dispatcher.dispatch(
                    type.__class__
                )(*args, **kw)
        wrapper.register = dispatcher.register
        update_wrapper(wrapper, func)
        return wrapper
    return _dimdispatch



import logging
from dataclasses import dataclass, field, InitVar
from typing import Dict, List, Optional, Tuple

from google.protobuf.internal import enum_type_wrapper

from .proto.common_types import common_types_pb2
from .qmpc_server import QMPCServer
from .share import Share
from .utils.parse_csv import (parse, parse_csv, parse_csv_to_bitvector,
                              parse_to_bitvector)
from .utils.restore import restore

logger = logging.getLogger(__name__)
# qmpc.JobStatus でアクセスできるようにエイリアスを設定する
JobStatus: enum_type_wrapper.EnumTypeWrapper \
    = common_types_pb2.JobStatus
ComputationMethod: enum_type_wrapper.EnumTypeWrapper \
    = common_types_pb2.ComputationMethod
JobErrorInfo = common_types_pb2.JobErrorInfo


@dataclass(frozen=True)
class QMPC:
    endpoints: InitVar[List[str]]
    # tokenがちゃんと使用されるようになったらデフォルト値を外す
    token: InitVar[str] = "token_demo"

    __qmpc_server: QMPCServer = field(init=False)
    __party_size: int = field(init=False)

    def __post_init__(self, endpoints: List[str],
                      token: str):
        logger.info(f"[QuickMPC server IP]={endpoints}")
        object.__setattr__(self, "_QMPC__qmpc_server", QMPCServer(
            endpoints, token))
        object.__setattr__(self, "_QMPC__party_size", len(endpoints))

    def parse_csv_file(self, filename: str) \
            -> Tuple[List[List[float]], List[str]]:
        logger.info("parse_csv_file. "
                    f"[filename]='{filename}'")
        return parse_csv(filename)

    def parse_csv_file_to_bitvector(self, filename: str,
                                    exclude: List[int] = []) \
            -> Tuple[List[List[float]], List[str]]:
        logger.info("parse_csv_file_to_bitvector. "
                    f"[filename]='{filename}' "
                    f"[not bitvector columns]={exclude}")
        return parse_csv_to_bitvector(filename, exclude)

    def parse_csv_data(self, data: List[List[str]]) \
            -> Tuple[List[List[float]], List[str]]:
        logger.info("parse_csv_data. "
                    f"[data size]={len(data)}x{len(data[0])}")
        return parse(data)

    def parse_csv_data_to_bitvector(self, data: List[List[str]],
                                    exclude: List[int] = []) \
            -> Tuple[List[List[float]], List[str]]:
        logger.info("parse_csv_file_to_bitvector. "
                    f"[data size]={len(data)}x{len(data[0])} "
                    f"[not bitvector columns]={exclude}")
        return parse_to_bitvector(data, exclude)

    def send_share(self, secrets: List, schema: List[str],
                   matching_column: int = 1,
                   piece_size: int = 1_000_000) -> Dict:
        logger.info("send_share request. "
                    f"[secrets size]={len(secrets)}x{len(secrets[0])} "
                    f"[matching ID name]={schema[matching_column-1]}")
        return self.__qmpc_server.send_share(
            secrets, schema, matching_column, piece_size)

    def delete_share(self, data_ids: List[str]) -> Dict:
        logger.info("delete_share request. "
                    f"[delete id list]={data_ids}")
        return self.__qmpc_server.delete_share(data_ids)

    def mean(self, join_order: Tuple[List, List, List],
             src: List) -> Dict:
        logger.info("mean request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]} "
                    f"[src columns]={src}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_MEAN"),
            join_order, (src, []))

    def variance(self, join_order: Tuple[List, List, List],
                 src: List) -> Dict:
        logger.info("variance request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]} "
                    f"[src columns]={src}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_VARIANCE"),
            join_order, (src, []))

    def sum(self, join_order: Tuple[List, List, List], src: List) -> Dict:
        logger.info("sum request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]} "
                    f"[src columns]={src}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_SUM"),
            join_order, (src, []))

    def correl(self, join_order: Tuple[List, List, List],
               inp: Tuple[List, List]) -> Dict:
        logger.info("correl request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]} "
                    f"[src columns]={inp[0]}"
                    f"[target columns]={inp[1]}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_CORREL"),
            join_order, inp)

    def linear_regression(self, join_order: Tuple[List, List, List],
                          inp: Tuple[List, List]) -> Dict:
        logger.info("linear_regression request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]} "
                    f"[design variable columns]={inp[0]}"
                    f"[objective variable columns]={inp[1]}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_LINEAR_REGRESSION"),
            join_order, inp)

    def logistic_regression(self, join_order: Tuple[List, List, List],
                            inp: Tuple[List, List]) -> Dict:
        logger.info("logistic_regression request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]} "
                    f"[design variable columns]={inp[0]}"
                    f"[objective variable columns]={inp[1]}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_LOGISTIC_REGRESSION"),
            join_order, inp)

    def meshcode(self, join_order: Tuple[List, List, List],
                 src: List) -> Dict:
        logger.info("meshcode request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]} "
                    f"[src columns]={src}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_MESH_CODE"),
            join_order, (src, []))

    def decision_tree(self, join_order: Tuple[List, List, List],
                      inp: Tuple[List, List]) -> Dict:
        logger.info("decision_tree request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]} "
                    f"[design variable columns]={inp[0]}"
                    f"[objective variable columns]={inp[1]}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_DECISION_TREE"),
            join_order, inp)

    def get_join_table(self, join_order: Tuple[List, List, List]) -> Dict:
        logger.info("get_join_table request. "
                    f"[data_id list]={join_order[0]} "
                    f"[join method]={join_order[1]} "
                    f"[matching ID columns]={join_order[2]}")
        return self.__qmpc_server.execute_computation(
            ComputationMethod.Value("COMPUTATION_METHOD_JOIN_TABLE"),
            join_order, (join_order[2], []))

    def get_computation_result(self, job_id: str,
                               path: Optional[str] = None) -> Dict:
        logger.info("get_computation_result request. "
                    f"[job_id]={job_id} "
                    f"[path]={path}")
        return self.__qmpc_server.get_computation_result(job_id, path)

    def get_data_list(self) \
            -> Dict:
        logger.info("get_data_list request.")
        return self.__qmpc_server.get_data_list()

    def demo_sharize(self, secrets: List) -> Dict:
        logger.info("demo_sharize request. "
                    f"[secrets size]={len(secrets)}x{len(secrets[0])}")
        share = Share.sharize(secrets, self.__party_size)
        return {'is_ok': True, 'results': share}

    def get_elapsed_time(self, job_id: str) -> Dict:
        logger.info("get_elapsed_time request. "
                    f"[job_id]={job_id}")
        return self.__qmpc_server.get_elapsed_time(job_id)

    def restore(self, job_id: str, path: str):
        logger.info("restore request. "
                    f"[job_id]={job_id} "
                    f"[path]={path}")
        return restore(job_id, path, self.__party_size)

    def get_job_error_info(self, job_id: str) -> Dict:
        logger.info("get_job_error_info request. "
                    f"[job_id]={job_id}")
        return self.__qmpc_server.get_job_error_info(job_id)

    @staticmethod
    def set_log_level(level: int):
        logger.setLevel(level)



import numpy as np

from quickmpc import QMPC


class TestQMPC:
    qmpc: QMPC = QMPC(
        ["http://localhost:50001",
         "http://localhost:50002",
         "http://localhost:50003"],
        "token_demo"
    )

    def test_restore(self, run_server1, run_server2, run_server3):
        """ファイルに書き出し、復元できるかテスト"""
        job_id: str = "uuid"
        path = "./tests/unit_tests/"
        res = self.qmpc.get_computation_result(job_id, path)
        assert (res["is_ok"])

        res = self.qmpc.restore(job_id, path)

        expect = [3.0, 6.0]
        assert (np.allclose(expect, res))



import csv
import glob
from decimal import Decimal
from typing import Any

import numpy as np
from natsort import natsorted


def get_meta(job_uuid: str, path: str):
    file_name = glob.glob(f"{path}/dim?-{job_uuid}-*")[0]
    with open(file_name, 'r') as f:
        reader = csv.reader(f)
        # 1列目のデータを取得
        meta = next(reader)
        return int(meta[0])


def get_result(job_uuid: str, path: str, party: int):
    for file_name in natsorted(glob.glob(f"{path}-{job_uuid}-{party}-*")):
        with open(file_name, 'r') as f:
            reader = csv.reader(f)
            # 1列目は読まない
            next(reader)
            for row in reader:
                for val in row:
                    yield val


def restore(job_uuid: str, path: str, party_size: int) -> Any:
    schema = []
    result: Any = []

    column_number = get_meta(job_uuid, path)
    is_dim2 = True if len(
        glob.glob(f"{path}/dim2-{job_uuid}-*")) != 0 else False
    if column_number == 0:
        return [[]] if is_dim2 else []

    for party in range(party_size):
        if party == 0:
            for val in get_result(job_uuid, f"{path}/schema", party):
                schema.append(val)

        itr = 0
        for val in get_result(job_uuid, f"{path}/dim?", party):
            if itr >= len(result):
                result.append(Decimal(val))
            else:
                result[itr] += Decimal(val)
            itr += 1

    result_float = np.vectorize(float)(result)
    result = np.array(result_float)\
        .reshape(-1, column_number).tolist() if is_dim2 else result_float
    result = {"schema": schema, "table": result} if len(schema) else result
    return result



import csv
import logging
from hashlib import sha512
from typing import List, Tuple

import numpy as np

logger = logging.getLogger(__name__)


def format_check(secrets: List[List[float]],
                 schema: List[str]) -> bool:
    # 存在チェック
    if not (schema and secrets):
        logger.error("Schema or secrets table are not exists.")
        return False
    # 重複チェック
    if len(schema) != len(set(schema)):
        logger.error("Duplicate schema name.")
        return False
    # サイズチェック
    if np.any([len(s) != len(schema) for s in secrets]):
        logger.error("schema size and table colummn size are different.")
        return False
    return True


def to_float(val: str) -> float:
    """ If val is a float, convert as is; if it is a string, hash it. """
    try:
        return float(val)
    except ValueError:
        # k,m are constants used in the comparison operation
        # Due to the limitation of comparison operation,
        # k bits are taken out and divided by 2^m.
        k: int = 48
        m: int = 20
        hs: str = sha512(val.encode()).hexdigest()
        val_int: int = int(hs[:(k >> 2)], 16)
        val_float: float = val_int / pow(2, m)
        return val_float


def parse(data: List[List[str]]) \
        -> Tuple[List[List[float]], List[str]]:
    schema: List[str] = data[0]
    secrets: List[List[float]] = [
        [to_float(x) for x in row] for row in data[1:]]

    if not format_check(secrets, schema):
        raise RuntimeError("規定されたフォーマットでないデータです．")

    return secrets, schema


def parse_to_bitvector(data: List[List[str]], exclude: List[int] = []) \
        -> Tuple[List[List[float]], List[str]]:
    secrets, schema = parse(data)

    secrets_bitbevtor: List[List[float]] = []
    schema_bitvector: List[str] = []
    for col, (sec, sch) in enumerate(zip(np.transpose(secrets), schema)):
        # 列が除外リストに含まれていたらそのままappend
        if col in exclude:
            secrets_bitbevtor.append(sec)
            schema_bitvector.append(sch+"#0")
            continue

        # 座標圧縮
        position: dict = {}
        it: int = 0
        for key in sec:
            if key not in position:
                position[key] = it
                it += 1

        # bitvector化
        for key, val in position.items():
            bitvector: list = [1 if (key == k) else 0 for k in sec]
            sch_val: str = sch+"#"+str(val)
            secrets_bitbevtor.append(bitvector)
            schema_bitvector.append(sch_val)

    return np.transpose(secrets_bitbevtor).tolist(), schema_bitvector


def parse_csv(filename: str) -> Tuple[List[List[float]], List[str]]:
    with open(filename) as f:
        reader = csv.reader(f)
        text: List[List[str]] = [row for row in reader]
        return parse(text)


def parse_csv_to_bitvector(filename: str, exclude: List[int] = []) ->  \
        Tuple[List[List[float]], List[str]]:
    with open(filename) as f:
        reader = csv.reader(f)
        text: List[List[str]] = [row for row in reader]
        return parse_to_bitvector(text, exclude)



from typing import Any, Dict

import pytest

from quickmpc.exception import ArgmentError, QMPCJobError, QMPCServerError
from quickmpc.qmpc_server import QMPCServer


class TestQMPC:
    qmpc: QMPCServer = QMPCServer(
        ["http://localhost:50001",
         "http://localhost:50002",
         "http://localhost:50003"],
        "token_demo"
    )

    @pytest.mark.parametrize(
        ("secrets", "schema"), [
            ([[1, 2, 3]], ["attr1", "attr2", "attr3"]),
            ([[1], [2], [3]], ["attr1"]), ]
    )
    def test_send_shares(self, secrets, schema,
                         run_server1, run_server2, run_server3):
        """ serverにシェアを送れるかのTest"""
        response: Dict[str, Any] = self.qmpc.send_share(
            secrets, schema, 1, 1000)
        assert (response["is_ok"])

    def test_send_shares_errorhandring(self,
                                       run_server1, run_server2, run_server3):
        with pytest.raises(RuntimeError):
            # piece_sizeが1000より小さい
            self.qmpc.send_share(
                [[1, 2, 3]], ["attr1", "attr2", "attr3"], 1, 500)
        with pytest.raises(RuntimeError):
            # piece_sizeが1000000より大きい
            self.qmpc.send_share(
                [[1, 2, 3]], ["attr1", "attr2", "attr3"], 1, 10000000)
        with pytest.raises(RuntimeError):
            # matching_columnが範囲外
            self.qmpc.send_share(
                [[1, 2, 3]], ["attr1", "attr2", "attr3"], -1, 1000)
        with pytest.raises(RuntimeError):
            # matching_columnが範囲外
            self.qmpc.send_share(
                [[1, 2, 3]], ["attr1", "attr2", "attr3"], 4, 1000)
        with pytest.raises(TypeError):
            # シェアがない
            self.qmpc.send_share(
                [], ["attr1", "attr2", "attr3"], 1, 1000)
        with pytest.raises(RuntimeError):
            # schemaに同じものが含まれる
            self.qmpc.send_share(
                [[1, 1, 2], [2, 3, 4]], ["attr1", "attr2", "attr2"], 1, 1000)
        with pytest.raises(RuntimeError):
            # 行によって列数が異なる
            self.qmpc.send_share(
                [[1, 1, 2], [2, 3, 4, 5]],
                ["attr1", "attr2", "attr3"], 1, 1000)

    def test_delete_shares(self, run_server1, run_server2, run_server3):
        """ serverにシェア削除要求を送れるかのTest"""
        response: Dict[str, Any] = self.qmpc.delete_share([])
        assert (response["is_ok"])

    def test_execute_computation(self, run_server1, run_server2, run_server3):
        """ serverに計算リクエストを送れるかのTest"""
        for method_id in range(1, 3):
            response: Dict[str, Any] = self.qmpc.execute_computation(
                method_id,
                [["id1", "id2"], [0], [1, 1]], [[0, 1], []])
            assert (response["is_ok"])

    def test_execute_computation_errorhandring(self, run_server1,
                                               run_server2, run_server3):
        with pytest.raises(ArgmentError):
            # data_idsの要素数-1とjoinの要素数が一致していない
            self.qmpc.execute_computation(
                1,
                [["id1", "id2"], [], [1, 1]], [[0, 1], []])
        with pytest.raises(ArgmentError):
            # data_idsの要素数とindexの要素数が一致していない
            self.qmpc.execute_computation(
                1,
                [["id1", "id2"], [0], [1]], [[0, 1], []])
        with pytest.raises(ArgmentError):
            # joinの値が0より小さい
            self.qmpc.execute_computation(
                1,
                [["id1", "id2"], [-1], [1, 1]], [[0, 1], []])
        with pytest.raises(ArgmentError):
            # joinの値が2より大きい
            self.qmpc.execute_computation(
                1,
                [["id1", "id2"], [3], [1, 1]], [[0, 1], []])

    def test_get_computation_resultRequest(self, run_server1,
                                           run_server2, run_server3):
        """ serverから結果を得られるかのTest """
        job_uuid: str = "uuid"
        response: Dict[str, Any] = self.qmpc.get_computation_result(
            job_uuid, None)
        assert (response["is_ok"])

    def test_exception_job_error(self):
        # QMPCJobErrorとして例外がthrowされるかのテスト
        with pytest.raises(QMPCJobError):
            self.qmpc.execute_computation(
                1,
                [["id1"], [], [1]], [[1000000000], []])

    def test_exception_server_error(self):
        # QMPCServerErrorとして例外がthrowされるかのテスト
        with pytest.raises(QMPCServerError):
            self.qmpc.execute_computation(
                1,
                [["UnregisteredDataId"], [], [1]], [[], []])

    def test_get_data_list(self, run_server1, run_server2, run_server3):
        """ serverにシェアを送れるかのTest"""
        response: Dict[str, Any] = self.qmpc.get_data_list()
        assert (response["is_ok"])

    def test_get_job_error_info(self, run_server1, run_server2, run_server3):
        # 例外がthrowされないことをテスト
        response: Dict[str, Any] = self.qmpc.get_job_error_info("test")

        assert (response["is_ok"])
        for res in response["job_error_info"]:
            assert (res.what == "QMPCJobError")
